import copy
from concurrent.futures import ThreadPoolExecutor, as_completed
import json
from collections import deque
import logging

from pmaw.PushshiftAPIBase import PushshiftAPIBase

log = logging.getLogger(__name__)


class PushshiftAPI(PushshiftAPIBase):
    def __init__(self, *args, **kwargs):
        """
        Helper class for interacting with the PushShift API for searching public reddit archival data using multiple threads.

        Input: 

            num_workers (int, optional) - Number of workers to use for multithreading, defaults to 10.
            max_sleep (int, optional) - Maximum rate-limit sleep time (in seconds) between requests, defaults to 60s.
            rate_limit (int, optional) - Target number of requests per minute for rate-averaging, defaults to 60 requests per minute.
            base_backoff (float, optional) - Base delay in seconds for exponential backoff, defaults to 0.5s
            batch_size (int, optional) - Size of batches for multithreading, defaults to number of workers.
            shards_down_behavior (str, optional) - Specifies how PMAW will respond if some shards are down during a query. Options are "warn" to only emit a warning, "stop" to throw a RuntimeError, or None to take no action. Defaults to "warn".
            limit_type (str, optional) - Type of rate limiting to use, default value is 'average' for rate averaging, use 'backoff' for exponential backoff
            jitter (str, optional) - Jitter to use with backoff, defaults to None, options are None, full, equal, decorr
            checkpoint (int, optional) - Size of interval in batches to print a checkpoint with stats, defaults to 10
            file_checkpoint (int, optional) - Size of interval in batches to cache responses when using mem_safe, defaults to 20
        """
        super().__init__(*args, **kwargs)

    def search_submission_comment_ids(self, ids, **kwargs):
        """
        Method for getting comment ids based on submission id(s)

        Input:
            ids (str, list) - Submission id(s) to return the comment ids of
            max_ids_per_request (int, optional) - Maximum number of ids to use in a single request, defaults to 1000, maximum 1000.
            mem_safe (boolean, optional) - If True, stores responses in cache during operation, defaults to False
            safe_exit (boolean, optional) - If True, will safely exit if interrupted by storing current responses and requests in the cache. Will also load previous requests / responses if found in cache, defaults to False
        Output:
            Response generator object
        """
        kwargs['ids'] = ids
        return self._search(kind='submission_comment_ids', **kwargs)

    def search_comments(self, **kwargs):
        """
        Method for searching comments, returns an array of comments

        Input:
            max_ids_per_request (int, optional) - Maximum number of ids to use in a single request, defaults to 1000, maximum 1000.
            max_results_per_request (int, optional) - Maximum number of items to return in a single non-id based request, defaults to 100, maximum 100.
            mem_safe (boolean, optional) - If True, stores responses in cache during operation, defaults to False
            search_window (int, optional) - Size in days for search window for submissions / comments in non-id based search, defaults to 365
            safe_exit (boolean, optional) - If True, will safely exit if interrupted by storing current responses and requests in the cache. Will also load previous requests / responses if found in cache, defaults to False
        Output:
            Response generator object
        """
        return self._search(kind='comment', **kwargs)

    def search_submissions(self, **kwargs):
        """
        Method for searching submissions, returns an array of submissions

        Input:
            max_ids_per_request (int, optional) - Maximum number of ids to use in a single request, defaults to 1000, maximum 1000.
            max_results_per_request (int, optional) - Maximum number of items to return in a single non-id based request, defaults to 100, maximum 100.
            mem_safe (boolean, optional) - If True, stores responses in cache during operation, defaults to False
            search_window (int, optional) - Size in days for search window for submissions / comments in non-id based search, defaults to 365
            safe_exit (boolean, optional) - If True, will safely exit if interrupted by storing current responses and requests in the cache. Will also load previous requests / responses if found in cache, defaults to False
        Output:
            Response generator object
        """
        return self._search(kind='submission', **kwargs)



import logging

log = logging.getLogger(__name__)


def timeslice(after, before, num):
    log.debug(
        f'Generating {num} slices between {after} and {before}')
    return [int((before-after)*i/num + after) for i in range(num+1)]


def mapslice(payload, after, before):
    payload['before'] = before
    payload['after'] = after
    return payload



import os
import pickle
import json
import hashlib
import logging
import re
import warnings

log = logging.getLogger(__name__)


class Cache(object):
    """Cache: Handle storing and loading request info and responses in the cache"""

    def __init__(self, payload, safe_exit):
        # generating key
        key_str = json.dumps(payload, sort_keys=True).encode("utf-8")
        self.key = hashlib.md5(key_str).hexdigest()
        print(f'Response cache key: {self.key}')

        # create cache folder
        self.folder = 'cache'
        try:
            os.mkdir(self.folder)
        except Exception as exc:
            log.debug(f'Folder creation failed - {exc}')

        self.response_cache = []
        self.size = 0
        if safe_exit:
            self.check_cache()

    def cache_responses(self, responses):
        if responses:
            num_resp = len(responses)
            checkpoint = len(self.response_cache) + 1
            self.size += num_resp
            print(
                f'File Checkpoint {checkpoint}:: Caching {num_resp} Responses')

            filename = f'{checkpoint}-{self.key}-{num_resp}.pickle'
            self.response_cache.append(filename)

            with open(f'./{self.folder}/{filename}', 'wb') as handle:
                pickle.dump(responses, handle,
                            protocol=pickle.HIGHEST_PROTOCOL)

    def load_info(self):
        try:
            with open(f'./{self.folder}/{self.key}_info.pickle', 'rb') as handle:
                return pickle.load(handle)
        except:
            log.info('No previous requests to load')

    def load_resp(self, cache_num):
        filename = self.response_cache[cache_num]
        try:
            with open(f'./{self.folder}/{filename}', 'rb') as handle:
                return pickle.load(handle)
        except Exception as exc:
            warnings.warn(f'Failed to load responses from {filename} - {exc}')

    def save_info(self, **kwargs):
        filename = f'./{self.folder}/{self.key}_info.pickle'
        with open(filename, 'wb') as handle:
            pickle.dump(kwargs, handle,
                        protocol=pickle.HIGHEST_PROTOCOL)

    def check_cache(self):
        for filename in os.listdir(self.folder):
            m = re.match(f"\d+-{self.key}-(\d+).pickle", filename)
            if m:
                self.response_cache.append(m.group(0))
                self.size += int(m.group(1))



# PMAW
# Copyright 2021 Matthew Podolak
# See LICENSE for details.

"""
PMAW: Pushshift Multithread API Wrapper
"""
__version__ = '1.0.5'
__author__ = 'Matthew Podolak'
__license__ = 'MIT'

from .RateLimit import RateLimit
from .Request import Request
from .Response import Response
from .Cache import Cache
from .PushshiftAPIBase import PushshiftAPIBase
from .PushshiftAPI import PushshiftAPI



import time
import datetime as dt
import requests
import json
from concurrent.futures import ThreadPoolExecutor, as_completed
import copy
import logging
import warnings

from pmaw.RateLimit import RateLimit
from pmaw.Request import Request
from pmaw.utils.slices import timeslice, mapslice

log = logging.getLogger(__name__)


class PushshiftAPIBase(object):
    _base_url = 'https://{domain}.pushshift.io/{{endpoint}}'

    def __init__(self, num_workers=10, max_sleep=60, rate_limit=60, base_backoff=0.5,
                 batch_size=None, shards_down_behavior='warn', limit_type='average', jitter=None,
                 checkpoint=10, file_checkpoint=20):
        self.num_workers = num_workers
        self.domain = 'api'
        self.shards_down_behavior = shards_down_behavior
        self.metadata_ = {}
        self.resp_dict = {}
        self.checkpoint = checkpoint
        self.file_checkpoint = file_checkpoint

        if batch_size:
            self.batch_size = batch_size
        else:
            self.batch_size = num_workers

        # instantiate rate limiter
        self._rate_limit = RateLimit(
            rate_limit, base_backoff, limit_type, max_sleep, jitter)

    @property
    def base_url(self):
        # getter for base_url, with formatted domain
        return self._base_url.format(domain=self.domain)

    def _impose_rate_limit(self):
        interval = self._rate_limit.delay()
        if interval > 0:
            time.sleep(interval)

    def _get(self, url, payload={}):
        self._impose_rate_limit()
        r = requests.get(url, params=payload)
        status = r.status_code
        reason = r.reason

        if status == 200:
            r = json.loads(r.text)

            # check if shards are down
            self.metadata_ = r.get('metadata', {})
            total_results = self.metadata_.get('total_results', None)
            if total_results:
                after, before = None, None
                for param in self.metadata_['ranges']:
                    created = param['range']['created_utc']
                    if created.get('gt', None):
                        after = created['gt']
                    elif created.get('lt', None):
                        before = created['lt']
                if after and before:
                    self.resp_dict[(after, before)] = total_results

            return r['data']
        else:
            raise Exception(f"HTTP {status} - {reason}")

    @property
    def shards_are_down(self):
        shards = self.metadata_.get('shards')
        if shards is None:
            return
        return shards['successful'] != shards['total']

    def _multithread(self, check_total=False):
        executor = ThreadPoolExecutor(max_workers=self.num_workers)

        while len(self.req.req_list) > 0 and not self.req.exit.is_set():
            # reset resp_dict which tracks remaining responses for timeslices
            self.resp_dict = {}

            # set number of futures created to batch size
            reqs = []
            if check_total:
                reqs.append(self.req.req_list.popleft())
            else:
                for i in range(min(len(self.req.req_list), self.batch_size)):
                    reqs.append(self.req.req_list.popleft())

            futures = {executor.submit(
                self._get, url_pay[0], url_pay[1]): url_pay for url_pay in reqs}

            self._futures_handler(futures, check_total)

            # reset attempts if no failures
            self._rate_limit._check_fail()

            # check if shards are down
            if self.shards_are_down and (self.shards_down_behavior is not None):
                shards_down_message = "Not all PushShift shards are active. Query results may be incomplete."
                if self.shards_down_behavior == 'warn':
                    log.warning(shards_down_message)
                if self.shards_down_behavior == 'stop':
                    self._shutdown(executor)
                    raise RuntimeError(
                        shards_down_message + f' {len(self.req.req_list)} unfinished requests.')
            if not check_total:
                self.num_batches += 1
                if self.num_batches % self.file_checkpoint == 0:
                    # cache current results
                    executor.submit(self.req.save_cache())
                self._print_stats('Checkpoint')
            else:
                break
        if not check_total:
            self._print_stats('Total')
        self._shutdown(executor)

    def _futures_handler(self, futures, check_total):
        for future in as_completed(futures):
            url_pay = futures[future]
            self.num_req += int(not check_total)
            try:
                data = future.result()
                self.num_suc += int(not check_total)
                url = url_pay[0]
                payload = url_pay[1]
                if not check_total:
                    self.req.save_resp(data)

                    log.debug(f'Remaining limit {self.req.limit}')
                    if self.req.limit <= 0:
                        log.debug(
                            f'Cancelling {len(self.req.req_list)} unfinished requests')
                        self.req.req_list.clear()
                        break

                    # handle time slicing logic
                    if 'before' in payload and 'after' in payload:
                        before = payload['before']
                        after = payload['after']
                        log.debug(
                            f"Time slice from {after} - {before} returned {len(data)} results")
                        total_results = self.resp_dict.get(
                            (after, before), 0)
                        log.debug(
                            f'{total_results} total results for this time slice')
                        # calculate remaining results
                        remaining = total_results - len(data)

                        # number of timeslices is depending on remaining results
                        if remaining > self.req.max_results_per_request*2:
                            num = 2
                        elif remaining > 0:
                            num = 1
                        else:
                            num = 0

                        if num > 0:
                            # find minimum `created_utc` to set as the `before` parameter in next timeslices
                            before = data[-1]['created_utc']

                            # generate payloads
                            self.req.gen_slices(
                                url, payload, after, before, num)
            except Exception as exc:
                log.debug(f"Request Failed -- {exc}")
                self._rate_limit._req_fail()
                self.req.req_list.appendleft(url_pay)

    def _shutdown(self, exc, wait=False, cancel_futures=True):
        # shutdown executor
        try:
            exc.shutdown(wait=wait, cancel_futures=cancel_futures)
        except:
            # TODO: manually cancel pending futures
            exc.shutdown(wait=wait)

    def _print_stats(self, prefix):
        rate = self.num_suc/self.num_req*100
        remaining = self.req.limit
        if (self.num_batches % self.checkpoint == 0) and prefix == 'Checkpoint':
            print(
                f'{prefix}:: Success Rate: {rate:.2f}% - Requests: {self.num_req} - Batches: {self.num_batches} - Items Remaining: {remaining}')
        elif prefix == 'Total':
            if remaining < 0:
                remaining = 0  # don't print a neg number
            print(
                f'{prefix}:: Success Rate: {rate:.2f}% - Requests: {self.num_req} - Batches: {self.num_batches} - Items Remaining: {remaining}')

    def _reset(self):
        self.num_suc = 0
        self.num_req = 0
        self.num_batches = 0

    def _search(self,
                kind,
                max_ids_per_request=1000,
                max_results_per_request=100,
                mem_safe=False,
                search_window=365,
                dataset='reddit',
                safe_exit=False,
                **kwargs):

        # raise error if aggs are requested
        if 'aggs' in kwargs:
            err_msg = "Aggregations support for {} has not yet been implemented, please use the PSAW package for your request"
            raise NotImplementedError(err_msg.format(kwargs['aggs']))

        self.metadata_ = {}
        self.req = Request(copy.deepcopy(kwargs), kind,
                           max_results_per_request, max_ids_per_request, mem_safe, safe_exit)

        # reset stat tracking
        self._reset()

        if kind == 'submission_comment_ids':
            endpoint = f'{dataset}/submission/comment_ids/'
        else:
            endpoint = f'{dataset}/{kind}/search'

        url = self.base_url.format(endpoint=endpoint)

        while (self.req.limit is None or self.req.limit > 0) and not self.req.exit.is_set():
            # set/update limit
            if 'ids' not in self.req.payload and len(self.req.req_list) == 0:
                # check to see how many results are remaining
                self.req.req_list.appendleft((url, self.req.payload))
                self._multithread(check_total=True)
                total_avail = self.metadata_.get('total_results', 0)

                if self.req.limit is None:
                    print(f'{total_avail} result(s) available in Pushshift')
                    self.req.limit = total_avail
                elif (total_avail < self.req.limit):
                    print(f'{self.req.limit - total_avail} result(s) not found in Pushshift')
                    self.req.limit = total_avail

            # generate payloads
            self.req.gen_url_payloads(
                url, self.batch_size, search_window)

            # check for exit signals
            self.req.check_sigs()

            if self.req.limit > 0 and len(self.req.req_list) > 0:
                self._multithread()

        self.req.save_cache()
        return self.req.resp



import logging
import copy
import datetime as dt
from collections import deque
import warnings

from pmaw.Cache import Cache
from pmaw.utils.slices import timeslice, mapslice
from pmaw.Response import Response
from threading import Event
import signal

log = logging.getLogger(__name__)


class Request(object):
    """Request: Handles request information, response saving, and cache usage."""

    def __init__(self, payload, kind, max_results_per_request, max_ids_per_request, mem_safe, safe_exit):
        self.kind = kind
        self.max_ids_per_request = min(1000, max_ids_per_request)
        self.max_results_per_request = min(100, max_results_per_request)
        self.safe_exit = safe_exit
        self.mem_safe = mem_safe
        self.req_list = deque()
        self.payload = payload
        self.limit = payload.get('limit', None)
        self.exit = Event()

        if 'ids' not in self.payload:
            # add necessary args
            self._add_nec_args(self.payload)

        if mem_safe or safe_exit:
            # instantiate cache
            _tmp = copy.deepcopy(payload)
            _tmp['kind'] = kind
            self._cache = Cache(_tmp, safe_exit)
            if safe_exit:
                info = self._cache.load_info()
                if info:
                    self.req_list.extend(info['req_list'])
                    self.payload = info['payload']
                    self.limit = info['limit']
                    print(
                        f'Loaded Cache:: Responses: {self._cache.size} - Pending Requests: {len(self.req_list)} - Items Remaining: {self.limit}')
        else:
            self._cache = None

        # instantiate response
        self.resp = Response(self._cache)

    def check_sigs(self):
        try:
            getattr(signal, 'SIGHUP')
            sigs = ('TERM', 'HUP', 'INT')
        except:
            sigs = ('TERM', 'INT')

        for sig in sigs:
            signal.signal(getattr(signal, 'SIG'+sig), self._exit)

    def save_cache(self):
        # trim extra responses
        self.trim()
        if self.safe_exit and self.limit and (self.limit == 0 or self.exit.is_set()):
            # save request info to cache
            self._cache.save_info(req_list=self.req_list,
                                  payload=self.payload, limit=self.limit)
            # save responses to cache
            self.resp.to_cache()
        elif self.mem_safe:
            self.resp.to_cache()

    def _exit(self, signo, _frame):
        self.exit.set()

    def save_resp(self, results):
        if self.kind == 'submission_comment_ids':
            self.limit -= 1
        else:
            self.limit -= len(results)
        self.resp.responses.extend(results)

    def _add_nec_args(self, payload):
        """Adds arguments to the payload as necessary."""

        payload['size'] = self.max_results_per_request

        if 'sort' not in payload:
            payload['sort'] = 'desc'
        elif payload.get('sort') != 'desc':
            err_msg = "Support for non-default sort has not been implemented as it may cause unexpected results"
            raise NotImplementedError(err_msg)

        if 'metadata' not in payload:
            payload['metadata'] = 'true'
        if 'before' not in payload:
            payload['before'] = int(dt.datetime.now().timestamp())
        if 'filter' in payload:
            if not isinstance(payload['filter'], list):
                if isinstance(payload['filter'], str):
                    payload['filter'] = [payload['filter']]
                else:
                    payload['filter'] = list(payload['filter'])
            # make sure that the created_utc field is returned
            if 'created_utc' not in payload['filter']:
                payload['filter'].append('created_utc')

    def gen_slices(self, url, payload, after, before, num):
        # create time slices
        ts = timeslice(after, before, num)
        url_payloads = [(url, mapslice(copy.deepcopy(payload),
                                       ts[i], ts[i+1])) for i in range(num)]
        self.req_list.extend(url_payloads)

    def gen_url_payloads(self, url, batch_size, search_window):
        """Creates a list of url payload tuples"""
        url_payloads = []

        # check if new payloads have to be made
        if len(self.req_list) == 0:
            # paging for ids
            if 'ids' in self.payload:

                # convert ids to list
                self._id_list(self.payload)

                all_ids = self.payload['ids']
                if len(all_ids) == 0 and (self.limit and self.limit > 0):
                    warnings.warn(
                        f'{self.limit} items were not found in Pushshift')                
                self.limit = len(all_ids)

                # remove ids from payload to prevent , -> %2C and increasing query length
                # beyond the max length of 8190
                self.payload['ids'] = []

                # if searching for submission comment ids
                if self.kind == "submission_comment_ids":
                    urls = [url+sub_id for sub_id in all_ids]
                    url_payloads = [(url, self.payload) for url in urls]
                else:
                    # split ids into arrays of size max_ids_per_request
                    ids_split = []
                    max_len = self.max_ids_per_request
                    while len(all_ids) > 0:
                        ids_split.append(",".join(all_ids[:max_len]))
                        all_ids = all_ids[max_len:]

                    log.debug(f'Created {len(ids_split)} id slices')

                    # create url payload tuples
                    url_payloads = [(url + '?ids=' + id_str, self.payload)
                                    for id_str in ids_split]
                # add payloads to req_list
                self.req_list.extend(url_payloads)

            else:
                if 'after' not in self.payload:
                    search_window = dt.timedelta(days=search_window)
                    num = batch_size
                    before = self.payload['before']
                    after = int((dt.datetime.fromtimestamp(
                        before) - search_window).timestamp())

                    # set before to after for future time slices
                    self.payload['before'] = after

                else:
                    before = self.payload['before']
                    after = self.payload['after']

                    # set before to avoid repeated time slices when there are missed responses
                    self.payload['before'] = after
                    num = batch_size

                # generate payloads
                self.gen_slices(
                    url, self.payload, after, before, num)

    def _id_list(self, payload):
        if not isinstance(payload['ids'], list):
            if isinstance(payload['ids'], str):
                payload['ids'] = [payload['ids']]
            else:
                payload['ids'] = list(payload['ids'])

    def trim(self):
        if self.limit and self.limit < 0:
            log.debug(f'Trimming {self.limit*-1} requests')
            self.resp.responses = self.resp.responses[:self.limit]



import logging
from collections.abc import Generator

log = logging.getLogger(__name__)


class Response(Generator):
    """Response: A generator which contains the responses from the request, loads from cache if needed."""

    def __init__(self, cache=None):
        self.responses = []
        self._cache = cache
        # track length of remainder
        self.num_returned = 0
        # indexing for returning responses
        self.i = 0
        self.num_cache = 0

    def to_cache(self):
        self._cache.cache_responses(self.responses)
        self.responses.clear()

    def _next_resp(self):
        resp = self.responses[self.i]
        self.i += 1
        return resp

    def send(self, ignored_arg):
        if self.i < len(self.responses):
            return self._next_resp()
        elif self._cache and self.num_cache < len(self._cache.response_cache):
            self.responses = self._cache.load_resp(self.num_cache)
            self.num_cache += 1
            # increase num returned to reflect responses retrieved from cache
            # as well as previously returned responses
            self.num_returned += (self.i + len(self.responses))
            self.i = 0
            return self._next_resp()
        else:
            self.responses.clear()
            raise StopIteration

    def __del__(self):
        self.close()

    def throw(self, type=None, value=None, traceback=None):
        log.debug('Cleaning up responses')
        self.responses.clear()
        raise StopIteration

    def __len__(self):
        if self._cache:
            length = len(self.responses) + self._cache.size - (self.i + self.num_returned) 
        else:
            length = len(self.responses) - self.i  
        
        return max(length, 0)



import logging
import time
import random

log = logging.getLogger(__name__)


class RateLimit(object):
    """RateLimit: Implements different rate-limiting strategies for concurrent requests"""

    def __init__(self, rate_limit=60, base_backoff=0.5, limit_type='average', max_sleep=60, jitter=None):
        self.rate_limit = rate_limit
        self.cache = list()
        self.base = base_backoff
        self.limit_type = limit_type
        self.max_sleep = max_sleep
        self.jitter = jitter
        self.sleep = self.base

        # track failures and attempts
        self.last_batch = 0
        self.attempts = 0
        self.num_fail = 0

    def delay(self):
        if self.limit_type:
            if self.limit_type == 'average':
                return min(self.max_sleep, self._average())
            elif self.limit_type == 'backoff':
                return self._backoff()
        else:
            return 0

    def _req_fail(self):
        self.num_fail += 1

    def _check_fail(self):
        # reset attempts if no new failures
        if self.last_batch == self.num_fail:
            self.num_fail = 0
            self.last_batch = 0
            self.attempts = 0
            self.sleep = self.base
        else:
            # store last batch num failures
            self.last_batch = self.num_fail

            # increase number of attempted batches
            self.attempts += 1

    def _expo(self):
        return min(self.max_sleep, self.base*pow(2, self.attempts))

    def _backoff(self):
        if self.jitter:
            if self.jitter == 'equal':
                v = self._expo()
                return v/2 + random.uniform(0, v/2)
            elif self.jitter == 'full':
                v = self._expo()
                return random.uniform(0, v)
            elif self.jitter == 'decorr':
                self.sleep = min(self.max_sleep, random.uniform(
                    self.base, self.sleep*3))
                return self.sleep
        else:
            return self._expo()

    def _average(self):
        # calculating delay required based on rate averaging
        curr_time = time.time()
        self.cache.append(curr_time)

        num_req = len(self.cache)
        first_req = min(self.cache)
        last_req = max(self.cache)

        # remove requests older than 60 seconds old
        while curr_time - first_req > 60:

            try:
                self.cache.remove(first_req)
            except:
                log.debug(f'{first_req} has already been removed RL cache')

            num_req = len(self.cache)
            first_req = min(self.cache)

        # return 0 if no other requests on cache
        if last_req == first_req:
            return 0
        else:
            period = last_req - first_req

            # project rate with no delay
            proj_rate = 60*(num_req)/(period)

            # check if projected rate is too high
            if(proj_rate < self.rate_limit or num_req < 5):
                return 0
            else:
                return 60*(num_req)/self.rate_limit - period



import setuptools
import re

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

VERSION_FILE = "pmaw/__init__.py"
with open(VERSION_FILE) as version_file:
    match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                      version_file.read(), re.MULTILINE)

if match:
    version = match.group(1)
else:
    raise RuntimeError(f"Unable to find version string in {VERSION_FILE}.")

setuptools.setup(
    name="pmaw",
    version=version,
    author="Matthew Podolak",
    author_email="mpodola2@gmail.com",
    description="A multithread Pushshift.io API Wrapper for reddit.com comment and submission searches.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mattpodolak/pmaw",
    packages=setuptools.find_packages(),
    license='MIT License',
    install_requires=['requests'],
    keywords='reddit api wrapper pushshift multithread data collection cache',
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3 :: Only",
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        "Intended Audience :: Developers",
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.5',
)
